//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.Zones
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			_calendar.InteractiveItemType = typeof(ZoneEvent);
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void _btnDefineZoneA_Click(object sender, System.EventArgs e)
		{
			AddZone(true);
		}

		private void _btnDefineZoneB_Click(object sender, System.EventArgs e)
		{
			AddZone(false);
		}

		private void _btnDeleteZone_Click(object sender, System.EventArgs e)
		{
			for (int j = 0; j < _zones.Count; )
			{
				Zone z = _zones[j] as Zone;

				// Check if zone is selected
				bool found = false;
				for (int i = 0; i < _calendar.Selection.Ranges.Count; i += 2)
				{
					DateTime start = (DateTime)_calendar.Selection.Ranges[i];
					DateTime end = (DateTime)_calendar.Selection.Ranges[i + 1];
					if (Intersect(z.Start, z.End, start, end))
					{
						_zones.RemoveAt(j);
						found = true;

						// Remove all items from the zone
						for (int k = 0; k < _calendar.Schedule.Items.Count; )
						{
							Item item = _calendar.Schedule.Items[k];

							if (Intersect(z.Start, z.End, item.StartTime, item.EndTime))
							{
								_calendar.Schedule.Items.RemoveAt(k);
								continue;
							}

							k++;
						}

						break;
					}
				}

				if (!found)
					j++;
			}

			_calendar.Selection.Reset();
		}

		private void _calendar_Draw(object sender, DrawEventArgs e)
		{
			if (e.Element == CustomDrawElements.TimetableCell)
			{
				DateTime cellStart = e.Date + e.StartTime;
				DateTime cellEnd = cellStart + _calendar.TimetableSettings.CellTime;
				bool zone = false;
				bool zoneStart = false;
				bool zoneEnd = false;
				bool type = false;

				foreach (Zone z in _zones)
				{
					if (z.Start == cellStart)
					{
						zone = true;
						type = z.Type;
						zoneStart = true;
					}

					if (z.End == cellEnd)
					{
						zone = true;
						type = z.Type;
						zoneEnd = true;
					}

					if (z.Start < cellStart && cellStart < z.End ||
						z.Start < cellEnd && cellEnd < z.End)
					{
						zone = true;
						type = z.Type;
					}
				}

				bool selected = false;
				if (_calendar.Selection.Contains(cellStart))
					selected = true;

				Rectangle b = e.Bounds;
				MindFusion.Drawing.IGraphics g = e.Graphics;
				Pen p = new Pen(e.Style.LineColor, 0);
				if (zone)
				{
					Brush b2 = new SolidBrush(Color.FromArgb(20, 255, 255, 255));
					Brush b4 = null;
					if (type)
						b4 = new SolidBrush(Color.FromArgb(75, Color.Goldenrod));
					else
						b4 = new SolidBrush(Color.FromArgb(75, Color.LightSteelBlue));

					g.FillRectangle(b2, b);
					g.FillRectangle(b4, b.Left + 1, b.Top, 30, b.Height);

					g.DrawLine(p, b.Left + 30, b.Top, b.Left + 30, b.Bottom - 1);

					g.DrawLine(p, b.Left, b.Top, b.Left, b.Bottom - 1);
					g.DrawLine(p, b.Right - 1, b.Top, b.Right - 1, b.Bottom - 1);

					if (zoneStart)
						g.DrawLine(p, b.Left, b.Top, b.Right - 1, b.Top);

					if (zoneEnd)
						g.DrawLine(p, b.Left, b.Bottom - 1, b.Right - 1, b.Bottom - 1);

					b2.Dispose();
					b4.Dispose();
				}

				Brush b1 = new SolidBrush(Color.FromArgb(40, 0, 0, 0));
				if (selected)
				{
					bool start = false;
					bool end = false;
					for (int i = 0; i < _calendar.Selection.Ranges.Count; i += 2)
					{
						DateTime istart = (DateTime)_calendar.Selection.Ranges[i];
						DateTime iend = (DateTime)_calendar.Selection.Ranges[i + 1];

						if (cellStart == istart)
							start = true;
						if (cellEnd == iend)
							end = true;
					}

					g.FillRectangle(b1, b);

					g.DrawLine(p, b.Left, b.Top, b.Left, b.Bottom - 1);
					g.DrawLine(p, b.Right - 1, b.Top, b.Right - 1, b.Bottom - 1);

					if (start)
						g.DrawLine(p, b.Left, b.Top, b.Right - 1, b.Top);

					if (end)
						g.DrawLine(p, b.Left, b.Bottom - 1, b.Right - 1, b.Bottom - 1);
				}
				b1.Dispose();

				p.Dispose();
			}
		}

		private void _calendar_ItemCreating(object sender, MindFusion.Scheduling.WinForms.ItemConfirmEventArgs e)
		{
			DateTime start = e.Item.StartTime;
			DateTime end = e.Item.EndTime;

			bool inZone = false;
			foreach (Zone z in _zones)
			{
				if (z.Start <= start && end <= z.End)
				{
					inZone = true;
					break;
				}
			}

			if (!inZone)
			{
				MessageBox.Show("Events cannot be created outside zones.");
				e.Confirm = false;
			}
		}

		private void _calendar_ItemModifying(object sender, MindFusion.Scheduling.WinForms.ItemModifyConfirmEventArgs e)
		{
			ZoneEvent item = e.Item as ZoneEvent;
			DateTime start = e.NewStartTime;
			DateTime end = e.NewEndTime;

			bool inZone = false;
			foreach (Zone z in _zones)
			{
				if (z.Start <= start && end <= z.End)
				{
					if (z.Type == item.ZoneType)
					{
						inZone = true;
						break;
					}
				}
			}

			if (!inZone)
			{
				e.Confirm = false;
			}
		}

		private void _calendar_ItemCreated(object sender, MindFusion.Scheduling.ItemEventArgs e)
		{
			ZoneEvent item = e.Item as ZoneEvent;

			foreach (Zone z in _zones)
			{
				if (Intersect(z.Start, z.End, item.StartTime, item.EndTime))
				{
					item.ZoneType = z.Type;
					break;
				}
			}
		}

		private void _calendar_ItemClick(object sender, MindFusion.Scheduling.WinForms.ItemMouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left &&
				e.Clicks == 2)
			{
				_calendar.ResetDrag();

				MessageBox.Show("This item is aligned to zone of type: " +
					((e.Item as ZoneEvent).ZoneType ? "'A'" : "'B'") + ".");
			}
		}


		private void AddZone(bool type)
		{
			if (_calendar.Selection.Ranges.Count < 2)
				return;

			DateTime start = (DateTime)_calendar.Selection.Ranges[0];
			DateTime end = (DateTime)_calendar.Selection.Ranges[1];

			// Check for zone intersection
			bool inter = false;
			foreach (Zone z in _zones)
			{
				if (Intersect(z.Start, z.End, start, end))
				{
					inter = true;
					break;
				}
			}

			if (inter)
			{
				MessageBox.Show("Cannot create intersecting zones.");
			}
			else
			{
				// Define a new zone
				_zones.Add(new Zone(start, end, "hey", type));

				_calendar.Selection.Reset();
			}
		}

		/// <summary>
		/// Checks whether the specified time intervals intersect.
		/// </summary>
		private static bool Intersect(
			DateTime start1, DateTime end1,
			DateTime start2, DateTime end2)
		{
			if (end2 <= start1 || end1 <= start2)
				return false;

			return true;
		}


		private string _appPath;

		/// <summary>
		/// A list with all defined zones.
		/// </summary>
		private ArrayList _zones = new ArrayList();
	}

	/// <summary>
	/// A definition of a zone.
	/// </summary>
	public class Zone
	{
		public Zone(DateTime start, DateTime end, string name, bool type)
		{
			_start = start;
			_end = end;
			_name = name;
			_type = type;
		}

		public DateTime Start
		{
			get { return _start; }
		}

		public DateTime End
		{
			get { return _end; }
		}

		public string Name
		{
			get { return _name; }
		}

		public bool Type
		{
			get { return _type; }
		}


		private DateTime _start;
		private DateTime _end;
		private string _name;
		private bool _type;
	}

	/// <summary>
	/// Our custom 'zoned' appointment.
	/// </summary>
	public class ZoneEvent : Appointment
	{
		/// <summary>
		/// Always define empty constructor for custom items.
		/// </summary>
		public ZoneEvent()
		{
			_zoneType = false;
		}

		/// <summary>
		/// Appointment.Clone override.
		/// </summary>
		public override object Clone()
		{
			ZoneEvent clone = new ZoneEvent();

			clone.AllDayEvent = this.AllDayEvent;
			clone.AllowChangeEnd = this.AllowChangeEnd;
			clone.AllowChangeStart = this.AllowChangeStart;
			clone.AllowMove = this.AllowMove;
			clone.DescriptionText = this.DescriptionText;
			clone.EndTime = this.EndTime;
			clone.HeaderText = this.HeaderText;
			clone.Location = this.Location;
			clone.Locked = this.Locked;
			clone.PointedSelectedStyle = this.PointedSelectedStyle.Clone() as Style;
			clone.PointedStyle = this.PointedStyle.Clone() as Style;
			clone.Priority = this.Priority;
			clone.Reminder = this.Reminder;
			clone.SelectedStyle = this.SelectedStyle.Clone() as Style;
			clone.StartTime = this.StartTime;
			clone.Style = this.Style.Clone() as Style;
			clone.Tag = this.Tag;
			clone.Task = this.Task;
			clone.Visible = this.Visible;

			foreach (Resource resource in this.Resources)
				clone.Resources.Add(resource);

			foreach (Contact contact in this.Contacts)
				clone.Contacts.Add(contact);

			clone.ZoneType = this.ZoneType;

			return clone;
		}


		/// <summary>
		/// Gets or sets the zone this item is aligned to.
		/// </summary>
		public bool ZoneType
		{
			get { return _zoneType; }
			set { _zoneType = value; }
		}


		private bool _zoneType;
	}
}
